#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "GrdAPI.h"

#define VERSION         1.0001

#define FRAME_LEN       95
#define FRAME_0         0
#define FRAME_1         9
#define FRAME_2         18
#define FRAME_3         27
#define FRAME_4         36
#define FRAME_5         46
#define FRAME_6         53
#define FRAME_7         61
#define FRAME_8         69
#define FRAME_9         78

#if defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64)
#define FRAME_V         0xB3
#define FRAME_H         0xC4
#define FRAME_LA1       0xDA
#define FRAME_RA1       0xBF
#define FRAME_HV1       0xC2
#define FRAME_VHL       0xC3
#define FRAME_VHR       0xB4
#define FRAME_X         0xC5
#define FRAME_LA2       0xC0
#define FRAME_HV2       0xC1
#define FRAME_RA2       0xD9

#elif defined(__GNUC__)
#define FRAME_V         ' '
#define FRAME_H         '-'
#define FRAME_LA1       '-'
#define FRAME_RA1       '-'
#define FRAME_HV1       '-'
#define FRAME_VHL       '-'
#define FRAME_VHR       '-'
#define FRAME_X         '-'
#define FRAME_LA2       '-'
#define FRAME_HV2       '-'
#define FRAME_RA2       '-'
#endif

#if defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64)
#define         stricmp_cross                           _stricmp
#elif defined(__GNUC__)
#define         stricmp_cross                           strcasecmp
#endif

#define NAME_BUF        50
#define TYPE_SI         GrdDT_
#define TYPE_SII        GrdDT_GSII64
#define TYPE_SIII       GrdDT_TRU
#define TYPE_TIME       GrdDT_RTC
#define TYPE_SIGN       GrdDT_AES
#define TYPE_CODE       GrdDT_LoadableCode

#define grdgf_ProtectTime               1
#define grdgf_HID                               2
#define grdgf_OnlyOneSessKey    4
#define grdgf_2ndSessKey                8

// Standard DEMO-strings
#define STR_DEMONVK "DEMONVK"           // DEMONVK = DEMONV1
#define STR_DEMORDO "DEMORDO"           // DEMORDO = DEMORD_
#define STR_DEMOPRF "DEMOPRF"           // DEMOPRF = DEMOPR5
#define STR_DEMOMST "DEMOMST"           // DEMOMST = DEMOMSY

#define nsk_MAX_CODE_LEN        6               // Number of essential symbols in code
#define nsk_MAX_CHARS           40              // Characters in one symbol

#define NULL    0

static  BYTE    _AL                             = 0;
HANDLE  hGrd;

// Search conditions
DWORD   dwRemoteMode							= GrdFMR_Local; // Operation mode flags
DWORD   dwFlags                                 = 0;                    // Operation mode flags
BYTE    byProg                                  = 0;                    // Program number
BYTE    byVer                                   = 0;                    // Version
DWORD   dwID                                    = 0;                    // Dongle ID
WORD    wSN                                     = 0;                    // Serial number
WORD    wMask                                   = 0;                    // Bit mask
WORD    wType                                   = 0;                    // Dongle type
DWORD   dwModel                                 = GrdFMM_ALL;   // Dongle model
DWORD   dwInterface                             = GrdFMI_ALL;   // Dongle interface type

typedef struct
{
	WORD    wYear;          // The year (1601 - 2099)
	WORD    wMonth;         // The month (January = 1, February = 2, ...)
	WORD    wDay;           // The day of the month (1-31)
	WORD    wHour;          // The hour (0-23)
	WORD    wMinute;        // The minute (0-59)
	WORD    wSecond;        // The second (0-59)
} SYSTEMTIME;

typedef struct
{
	unsigned    mnft_tseconds : 5;      // Two seconds (seconds*2)
	unsigned    mnft_minutes : 6;       // Minuts
	unsigned    mnft_hours : 5;         // Hours
	unsigned    mnft_day : 5;           // Days
	unsigned    mnft_month : 4;         // Months
	unsigned    mnft_year : 7;          // Year - 1990
} GUARDANTDONGLEID, *PGUARDANTDONGLEID;

///////////////////////////////////////////////////////////////////////////////////////////
// Frame customization function
void MakeFrame(int nType, char* szFrame)
{
	BYTE        byCount = 0;

	if (nType == 1)
	{
		for (byCount = 0; byCount < FRAME_9; ++byCount)
			szFrame[byCount] = FRAME_H;

		szFrame[FRAME_0] = FRAME_LA1;
		szFrame[FRAME_5] = FRAME_HV1;
		szFrame[FRAME_9] = FRAME_RA1;
		szFrame[FRAME_9 + 1] = 0x00;
	}

	if (nType == 2)
	{
		for (byCount = 0; byCount < 80; ++byCount)
			szFrame[byCount] = FRAME_H;

		szFrame[FRAME_0] = FRAME_VHL;
		szFrame[FRAME_1] = FRAME_HV1;
		szFrame[FRAME_2] = FRAME_HV1;
		szFrame[FRAME_3] = FRAME_HV1;
		szFrame[FRAME_4] = FRAME_HV1;
		szFrame[FRAME_5] = FRAME_X;
		szFrame[FRAME_6] = FRAME_HV1;
		szFrame[FRAME_7] = FRAME_HV1;
		szFrame[FRAME_8] = FRAME_HV1;
		szFrame[FRAME_9] = FRAME_VHR;
		szFrame[FRAME_9 + 1] = 0x00;
	}

	if (nType == 3)
	{
		for (byCount = 0; byCount < 80; ++byCount)
			szFrame[byCount] = FRAME_H;

		szFrame[FRAME_0] = FRAME_VHL;
		szFrame[FRAME_1] = FRAME_X;
		szFrame[FRAME_2] = FRAME_X;
		szFrame[FRAME_3] = FRAME_X;
		szFrame[FRAME_4] = FRAME_X;
		szFrame[FRAME_5] = FRAME_X;
		szFrame[FRAME_6] = FRAME_X;
		szFrame[FRAME_7] = FRAME_X;
		szFrame[FRAME_8] = FRAME_X;
		szFrame[FRAME_9] = FRAME_VHR;
		szFrame[FRAME_9 + 1] = 0x00;
	}

	if (nType == 4)
	{
		for (byCount = 0; byCount < 80; ++byCount)
			szFrame[byCount] = FRAME_H;

		szFrame[FRAME_0] = FRAME_LA2;
		szFrame[FRAME_1] = FRAME_HV2;
		szFrame[FRAME_2] = FRAME_HV2;
		szFrame[FRAME_3] = FRAME_HV2;
		szFrame[FRAME_4] = FRAME_HV2;
		szFrame[FRAME_5] = FRAME_HV2;
		szFrame[FRAME_6] = FRAME_HV2;
		szFrame[FRAME_7] = FRAME_HV2;
		szFrame[FRAME_8] = FRAME_HV2;
		szFrame[FRAME_9] = FRAME_RA2;
		szFrame[FRAME_9 + 1] = 0x00;
	}

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////
// Add nuber of spaces before szDest[dwLength] symbol, set it to chTail and cut string by 0
//      Input:
//              szDest - destination buf
//              dwLength - new string length
//              chTail - last character
void FrameCorrection(char* szDest, int nLength, char chTail)
{
	int i;

	for (i = strlen(szDest); i < nLength; ++i)
		szDest[i] = ' ';

	szDest[nLength] = chTail;
	szDest[nLength + 1] = '\0';
	return;
}

///////////////////////////////////////////////////////////////////////////////////////////
// Convert BYTE from CODE to ASCII NSK CODE
//      Input:
//              byC  - BYTE BINARY CODE
//      Return:
//              ASCII symbol
BYTE bin2ascNSK(BYTE byC)
{
	_AL = byC;

	if (_AL <= 9)
		return('0' + _AL);

	if (_AL >= 10 && _AL < 36)
		return(('A' - 10) + _AL);

	if (_AL == 36)
		return('+');

	if (_AL == 37)
		return('-');

	if (_AL == 38)
		return('&');

	return('_');
}
///////////////////////////////////////////////////////////////////////////////////////////
// Convert LONG BINARY CODE to ASCIIZ NSK CODE
//      Input:
//              dwAsc  - LONG BINARY CODE
//              pszAsc - ASCIIZ NSK CODE
void bin2asczNSK(DWORD dwAsc, void* pszAscz)
{

#if defined(__GNUC__)
	WORD  i = nsk_MAX_CODE_LEN;
	BYTE cb = 0xFF, *p = (BYTE*)pszAscz;
	p += i;
	*p = 0;

	do
	{
		_AL = dwAsc % nsk_MAX_CHARS;
		*--p = bin2ascNSK((BYTE)_AL);
		dwAsc /= nsk_MAX_CHARS;
	}
	while ((--i) != 0);

	i = nsk_MAX_CODE_LEN;
	asm("clc");
	asm("pushf");

	do
	{
		asm("popf");
		asm("rolb $1, %0": :"m"(cb));
		asm("rolb $1, %0": :"m"(cb));
		asm("pushf");
		cb += *p++;
	}
	while ((--i) != 0);

	asm("popf");
	_AL = cb % nsk_MAX_CHARS;
	*p++ = bin2ascNSK((BYTE)_AL);
	*p = 0;

#elif defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64)
	WORD  i = nsk_MAX_CODE_LEN;
	BYTE cb = 0xFF, *p = (BYTE*)pszAscz;
	p += i;
	*p = 0;

	do
	{
		_AL = dwAsc % nsk_MAX_CHARS;
		*--p = bin2ascNSK((BYTE)_AL);
		dwAsc /= nsk_MAX_CHARS;
	}
	while ((--i) != 0);

	i = nsk_MAX_CODE_LEN;
	__asm clc
	__asm pushf

	do
	{
		__asm popf
		__asm rol byte ptr cb, 1
		__asm rol byte ptr cb, 1
		__asm pushf
		cb += *p++;
	}
	while ((--i) != 0);

	__asm popf
	_AL = cb % nsk_MAX_CHARS;
	*p++ = bin2ascNSK((BYTE)_AL);
	*p = 0;
#endif
}
///////////////////////////////////////////////////////////////////////////////////////////
// Convert LONG BINARY public CODE (LpuBC) to ASCIIZ public CODE (ApuC)
//      Input:
//              CODE-LONG BINARY CODE (LBpuC)
//      Return:
//              pointer to ASCIIZ CODE (ApuC) static buffer
char* GetPublicStringNSK(DWORD CODE)
{
	static char PublicCODEs[10];

	switch (CODE)
	{
		case GrdDC_DEMONVK :
			strcpy(PublicCODEs, STR_DEMONVK);
			break;

		case GrdDC_DEMORDO :
			strcpy(PublicCODEs, STR_DEMORDO);
			break;

		case GrdDC_DEMOPRF :
			strcpy(PublicCODEs, STR_DEMOPRF);
			break;

		case GrdDC_DEMOMST :
			strcpy(PublicCODEs, STR_DEMOMST);
			break;

		default :
			bin2asczNSK(CODE, PublicCODEs);
			break;
	}

	return(PublicCODEs);
}
///////////////////////////////////////////////////////////////////////////////////////////
// Convert byMaxNetRes and wRealNetRes fields of GrdFindInfo to Dongle real LAN resource
//      Input:
//              byMaxNetRes - Hardware LAN resource
//              wRealNetRes - Software LAN resource
//      Return:
//              real Dongle LAN resource
DWORD GetRealLANResource(BYTE byMaxNetRes, WORD wRealNetRes)
{
	DWORD  dwHardwareLanRes = byMaxNetRes;
	DWORD  dwGP2Licences = wRealNetRes;
	DWORD  dwMaxLicences = 0;

	switch (dwHardwareLanRes)
	{
		case 253:
			dwMaxLicences = 500;
			break;

		case 254:
			dwMaxLicences = 1000;
			break;

		case 255:
			dwMaxLicences = -1;
			break;      // Unlimited

		default:
			dwMaxLicences = dwHardwareLanRes;
			break;
	}

	if (dwMaxLicences >= 0)
	{
		// Not unlimited dongles
		if (dwGP2Licences < dwMaxLicences)
			dwMaxLicences = dwGP2Licences;
	}
	else
	{
		// Unlimited dongles
		if (dwGP2Licences != 0xFFFF)
			dwMaxLicences = dwGP2Licences;
	}

	return dwMaxLicences;
}
///////////////////////////////////////////////////////////////////////////////////////////
// Parse GrdFindInfo structure
// Print full dongle name into ASCIIZ (szName)
//      Input:
//              szName - buffer pointer
//              GrdFindInfo - GrdFindInfo pointer
void NameResolution(char* szName, TGrdFindInfo* GrdFindInfo)
{
	strcpy(szName, " Guardant ");

	if (GrdFindInfo->wType & TYPE_CODE && GrdFindInfo->wType & TYPE_TIME)
	{
		strcpy(szName + strlen(szName), "Code Time ");

		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net ");
	}
	else if (GrdFindInfo->wType & TYPE_CODE)
	{
		strcpy(szName + strlen(szName), "Code ");

		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net ");
	}
	else if (GrdFindInfo->wType & TYPE_TIME)
	{
		strcpy(szName + strlen(szName), "Time ");

		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net ");
	}
	else if (GrdFindInfo->wType & TYPE_SIGN)
	{
		strcpy(szName + strlen(szName), "Sign ");

		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net ");
	}
	else if (GrdFindInfo->wType & TYPE_SIII)
	{
		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net III ");
		else
			strcpy(szName + strlen(szName), "Stealth III ");
	}
	else if (GrdFindInfo->wType & TYPE_SII)
	{
		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net II ");
		else
			strcpy(szName + strlen(szName), "Stealth II ");
	}
	else if (GrdFindInfo->dwModel == GrdDM_GF1U || GrdFindInfo->dwModel == GrdDM_GF1L)
	{
		strcpy(szName + strlen(szName), "Fidus ");
	}
	else if (GrdFindInfo->dwModel == GrdDM_GS1L || GrdFindInfo->dwModel == GrdDM_GS1U)
	{
		if (GrdFindInfo->byMaxNetRes != 0 && GrdFindInfo->wType & GrdDT_LAN)
			strcpy(szName + strlen(szName), "Net I ");
		else
			strcpy(szName + strlen(szName), "Stealth I ");
	}
	else
		strcpy(szName + strlen(szName), "Unknown type ");

	if (GrdFindInfo->dwModel == 0 || GrdFindInfo->dwModel == 2 || GrdFindInfo->dwModel == 3)
		strcpy(szName + strlen(szName), "LPT ");

	strcpy(szName + strlen(szName), "Dongle ");
};
///////////////////////////////////////////////////////////////////////////////////////////
// Parse 'wType' field of GrdFindInfo structure
// Print DONGLE TYPE description into ASCIIZ (szType)
//      Input:
//              szType - buffer pointer
//              wType - WORD field wType of GrdFindInfo
void TypeResolution(char* szType, WORD wType)
{
	memset(szType, 0, NAME_BUF);

	if (wType & GrdDT_DOS)
	{
		strcpy(szType, "DOS");
	}

	if (wType & GrdDT_Win)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "WIN");
	}

	if (wType & GrdDT_LAN)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "LAN");
	}

	if (wType & GrdDT_Time)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "Time");
	}

	if (wType & GrdDT_GSII64)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "GSII64");
	}

	if (wType & GrdDT_PI)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "PI");
	}

	if (wType & GrdDT_TRU)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "TRU");
	}

	if (wType & GrdDT_RTC)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "RTC");
	}

	if (wType & GrdDT_AES)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "AES");
	}

	if (wType & GrdDT_ECC)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "ECC");
	}

	if (wType & GrdDT_LoadableCode)
	{
		if (strlen(szType)) strcpy(szType + strlen(szType), ",");

		strcpy(szType + strlen(szType), "CODE");
	}
}
///////////////////////////////////////////////////////////////////////////////////////////
// Parse 'wGlobalFlags' field of GrdFindInfo structure
// Print GLOBAL FLAGS description into ASCIIZ (szGF)
//      Input:
//              szGF - buffer pointer
//              wGlobalFlags - WORD field wType of GrdFindInfo
void GfResolution(char* szGF, WORD wGlobalFlags)
{
	memset(szGF, 0, NAME_BUF);

	if (wGlobalFlags == 0)
	{
		strcpy(szGF, "NONE");
		return;
	}

	if (wGlobalFlags & grdgf_ProtectTime)
	{
		if (strlen(szGF)) strcpy(szGF + strlen(szGF), ",");

		strcpy(szGF + strlen(szGF), "PTIME");
	}

	if (wGlobalFlags & grdgf_HID)
	{
		if (strlen(szGF)) strcpy(szGF + strlen(szGF), ",");

		strcpy(szGF + strlen(szGF), "HID");
	}

	if (wGlobalFlags & grdgf_OnlyOneSessKey)
	{
		if (strlen(szGF)) strcpy(szGF + strlen(szGF), ",");

		strcpy(szGF + strlen(szGF), "API_SK");
	}

	if (wGlobalFlags & grdgf_2ndSessKey)
	{
		if (strlen(szGF)) strcpy(szGF + strlen(szGF), ",");

		strcpy(szGF + strlen(szGF), "AUTOPROT_SK");
	}
}
///////////////////////////////////////////////////////////////////////////////////////////
// Parse 'ID' field of GrdFindInfo structure
// Fill SYSTEMTIME(st) structure with the Manufacture Date
//      Input:
//              st - Destination SYSTEMTIME structure pointer
//              dwID - ID field of GrdFindInfo structure
void GetManufactureDate(SYSTEMTIME* MTime, DWORD dwID)
{
	PGUARDANTDONGLEID pID = (PGUARDANTDONGLEID)&dwID;

	memset(MTime, 0, sizeof(SYSTEMTIME));

	MTime->wYear = (WORD)pID->mnft_year + 1990;
	MTime->wMonth = (WORD)pID->mnft_month;
	MTime->wDay = (WORD)pID->mnft_day;
	MTime->wHour = (WORD)pID->mnft_hours;
	MTime->wMinute = (WORD)pID->mnft_minutes;
	MTime->wSecond = (WORD)pID->mnft_tseconds * 2;
}
///////////////////////////////////////////////////////////////////////////////////////////
// Print result of GrdAPI function execution
//      Input:
//              nErrorCode - function returned value
//      Rerurn:
//              nErrorCode - function returned value
int     PrintResult(int nErrorCode)
{
	char        szErrorMsg[0x100];      // Buffer for error string
	int         nRet;
	nRet = GrdFormatMessage(
			NULL,                                    // Handle may by closed on this time
			nErrorCode,                              // Guardant API error code
			GrdLng_ENG,                              // Request english string
			szErrorMsg,                              // Place for error string
			sizeof(szErrorMsg),                       // Max buffer size
			NULL);                                   // Reserved

	if (nErrorCode == GrdE_OK) printf(szErrorMsg);
	else printf("\nError! %s.\n", szErrorMsg);

	return nErrorCode;
};

///////////////////////////////////////////////////////////////////////////////////////////
// Collect info about Dongles and print into standart output
int Search4Dongles()
{
	int         nRet = 0;
	int         nDonglesFound = 0;
	char        szDongleName[NAME_BUF];
	char        szFrame[FRAME_LEN];
	char        szBuf1[FRAME_LEN];
	char        szBuf2[FRAME_LEN];
	char        szType[NAME_BUF];
	char        szGF[NAME_BUF];
	char        chLine = FRAME_V;
	SYSTEMTIME sDateTime;
	TGrdFindInfo GrdFindInfo;

	nRet = GrdStartup(dwRemoteMode);

	if (nRet)PrintResult(nRet);

	hGrd = GrdCreateHandle(NULL, GrdCHM_SingleThread, NULL);
	nRet = GrdSetFindMode(hGrd, dwRemoteMode, dwFlags, byProg, dwID, wSN, byVer, wMask, wType, dwModel, dwInterface);

	if (nRet)PrintResult(nRet);

	nRet = GrdFind(hGrd, GrdF_First, &dwID, &GrdFindInfo);

	printf("\n");

	while (!nRet)
	{
		MakeFrame(1, szFrame);
		NameResolution(szDongleName, &GrdFindInfo);
		strncpy(szFrame + 13, szDongleName, strlen(szDongleName));
		printf("%s\n", szFrame);

		GetManufactureDate(&sDateTime, GrdFindInfo.dwID);
		sprintf(szBuf1, "%c Produce Date (dd.mm.yy):", chLine);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %02d.%02d.%04d (ID=0x%08X)", szBuf1, sDateTime.wDay, sDateTime.wMonth, sDateTime.wYear, GrdFindInfo.dwID);
		FrameCorrection(szBuf2, FRAME_9, chLine);
		printf("%s\n", szBuf2);

		sprintf(szBuf1, "%c Versions (Hrw,MCUF,Type,PrtSup,Std,Cl,DSt):", chLine);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %X.%X,%02X,%02X,%02X,%02X,%X.%X,%X", szBuf1,
				(BYTE)(GrdFindInfo.byHrwVersion / 16), GrdFindInfo.byHrwVersion % 16,
				GrdFindInfo.dwRFlags, GrdFindInfo.wType, GrdFindInfo.dwPrnPort, GrdFindInfo.dwDongleState,
				(BYTE)(GrdFindInfo.dwNSKClientVersion / 16), GrdFindInfo.dwNSKClientVersion % 16,
				GrdFindInfo.dwDongleState);
		FrameCorrection(szBuf2, FRAME_9, chLine);
		printf("%s\n", szBuf2);

		TypeResolution(szType, GrdFindInfo.wType);
		sprintf(szBuf1, "%c Supports:", chLine);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %s", szBuf1, szType);
		FrameCorrection(szBuf2, FRAME_9, chLine);
		printf("%s\n", szBuf2);

		GfResolution(szGF, GrdFindInfo.wGlobalFlags);
		sprintf(szBuf1, "%c Global Flags:", chLine);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %s", szBuf1, szGF);
		FrameCorrection(szBuf2, FRAME_9, chLine);
		printf("%s\n", szBuf2);

		sprintf(szBuf1, "%c Public Code:", chLine);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %s (0x%08X)", szBuf1, GetPublicStringNSK(GrdFindInfo.dwPublicCode), GrdFindInfo.dwPublicCode);
		FrameCorrection(szBuf2, FRAME_9, chLine);
		printf("%s\n", szBuf2);

		MakeFrame(2, szFrame);
		printf("%s\n", szFrame);

		sprintf(szBuf1, "%cFirmware", chLine);
		FrameCorrection(szBuf1, FRAME_1, chLine);
		sprintf(szBuf2, "%s%s", szBuf1, " ProgNum");
		FrameCorrection(szBuf2, FRAME_2, chLine);
		sprintf(szBuf1, "%s%s", szBuf2, " Version");
		FrameCorrection(szBuf1, FRAME_3, chLine);
		sprintf(szBuf2, "%s%s", szBuf1, " SerialN");
		FrameCorrection(szBuf2, FRAME_4, chLine);
		sprintf(szBuf1, "%s%s", szBuf2, "GPcounter");
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s%s", szBuf1, " Mask");
		FrameCorrection(szBuf2, FRAME_6, chLine);
		sprintf(szBuf1, "%s%s", szBuf2, " NetRes");
		FrameCorrection(szBuf1, FRAME_7, chLine);
		sprintf(szBuf2, "%s%s", szBuf1, "RealRes");
		FrameCorrection(szBuf2, FRAME_8, chLine);
		sprintf(szBuf1, "%s%s", szBuf2, " Index");
		FrameCorrection(szBuf1, FRAME_9, chLine);
		printf("%s\n", szBuf1);

		MakeFrame(3, szFrame);
		printf("%s\n", szFrame);

		sprintf(szBuf1, "%c%01X.%01X.%01X.%02X", chLine,
				GrdFindInfo.dwRProgVer / (256 * 256 * 256),
				(GrdFindInfo.dwRProgVer / (256 * 256)) % 256,
				(GrdFindInfo.dwRProgVer / 256) % 256,
				GrdFindInfo.dwRProgVer % 256);
		FrameCorrection(szBuf1, FRAME_1, chLine);
		sprintf(szBuf2, "%s%8d", szBuf1, GrdFindInfo.byNProg);
		FrameCorrection(szBuf2, FRAME_2, chLine);
		sprintf(szBuf1, "%s%8d", szBuf2, GrdFindInfo.byVer);
		FrameCorrection(szBuf1, FRAME_3, chLine);
		sprintf(szBuf2, "%s%8d", szBuf1, GrdFindInfo.wSN);
		FrameCorrection(szBuf2, FRAME_4, chLine);
		sprintf(szBuf1, "%s%9d", szBuf2, GrdFindInfo.wGP);
		FrameCorrection(szBuf1, FRAME_5, chLine);
		sprintf(szBuf2, "%s %04Xh", szBuf1, GrdFindInfo.wMask);
		FrameCorrection(szBuf2, FRAME_6, chLine);
		sprintf(szBuf1, "%s%7d", szBuf2, GrdFindInfo.byMaxNetRes);
		FrameCorrection(szBuf1, FRAME_7, chLine);
		sprintf(szBuf2, "%s%7d", szBuf1, GetRealLANResource(GrdFindInfo.byMaxNetRes, GrdFindInfo.wRealNetRes));
		FrameCorrection(szBuf2, FRAME_8, chLine);
		sprintf(szBuf1, "%s%08X", szBuf2, GrdFindInfo.dwIndex);
		FrameCorrection(szBuf1, FRAME_9, chLine);
		printf("%s\n", szBuf1);

		MakeFrame(4, szFrame);
		printf("%s\n", szFrame);

		nDonglesFound++;
		nRet = GrdFind(hGrd, GrdF_Next, &dwID, &GrdFindInfo);
	}

	GrdCloseHandle(hGrd);
	GrdCleanup();
	return nDonglesFound;
};
///////////////////////////////////////////////////////////////////////////////////////////
// Main
int main(int argc, char* argv[])
{
	int i = 0;
	int nDonglesFound = 0;

	printf("\nWelcome to Guardant Diagnostic Tool! (v%01.04f)\t\t", VERSION);

	for (i = 0; i < argc; ++i)
	{
		if (!stricmp_cross(argv[i], "net") || !stricmp_cross(argv[i], "remote"))
			dwRemoteMode = GrdFMR_Remote;
	}

	printf("\n\nDetecting Guardant dongles ...\n");
	nDonglesFound = Search4Dongles();

	printf("Result: %d dongle(s) found. Press Enter...", nDonglesFound);
	getchar();

	return 0;
}
